/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ScanClassVisitor.java 2483 2008-02-20 09:02:57Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.scan.impl;

import java.util.ArrayList;
import java.util.List;

import org.ow2.util.asm.ClassVisitor;
import org.ow2.util.asm.FieldVisitor;
import org.ow2.util.asm.MethodVisitor;
import org.ow2.util.scan.api.configurator.IArchiveConfigurator;
import org.ow2.util.scan.api.configurator.IClassConfigurator;
import org.ow2.util.scan.api.configurator.IFieldConfigurator;
import org.ow2.util.scan.api.configurator.IMethodConfigurator;
import org.ow2.util.scan.api.metadata.structures.JClass;
import org.ow2.util.scan.api.metadata.structures.JField;
import org.ow2.util.scan.api.metadata.structures.JMethod;

/**
 * ScanClassVisitor.
 * @author Gael Lalire
 */
public final class ScanClassVisitor extends ScanCommonVisitor<IClassConfigurator> implements ClassVisitor {

    /**
     * list of archives.
     */
    private List<IArchiveConfigurator> archiveConfigurators;

    /**
     * Constructor.
     * @param archiveConfigurators list of archives
     */
    public ScanClassVisitor(final List<IArchiveConfigurator> archiveConfigurators) {
        this.archiveConfigurators = archiveConfigurators;
    }

    /**
     * visit the class.
     * @param version the version
     * @param access access (public, ...)
     * @param name class name
     * @param signature signature
     * @param superName super class name
     * @param interfaces implemented interfaces name
     */
    @Override
    public void visit(final int version,
                      final int access,
                      final String name,
                      final String signature,
                      final String superName,
                      final String[] interfaces) {
        List<IClassConfigurator> classConfigurators = new ArrayList<IClassConfigurator>();
        JClass jClass = new JClass(version, access, name, signature, superName, interfaces);
        for (IArchiveConfigurator archiveConfigurator : archiveConfigurators) {
            classConfigurators.add(archiveConfigurator.createClassConfigurator(jClass));
        }
        setCommonConfigurators(classConfigurators);
    }

    /**
     * @param access access of method (public, ...)
     * @param name name of method
     * @param desc desc
     * @param signature signature of method
     * @param exceptions throwed exceptions
     * @return a visitor
     */
    @Override
    public MethodVisitor visitMethod(final int access, final String name, final String desc, final String signature,
            final String[] exceptions) {
        JMethod jMethod = new JMethod(access, name, desc, signature, exceptions);
        List<IMethodConfigurator> methodConfigurators = new ArrayList<IMethodConfigurator>();
        for (IClassConfigurator classConfigurator : getCommonConfigurators()) {
            methodConfigurators.add(classConfigurator.createMethodConfigurator(jMethod));
        }
        return new ScanMethodVisitor(methodConfigurators);
    }

    /**
     * @param access access of field (public, ...)
     * @param name name of field
     * @param desc desc
     * @param signature signature of field
     * @param value value of field
     * @return a visitor
     */
    @Override
    public FieldVisitor visitField(final int access, final String name, final String desc, final String signature,
            final Object value) {
        JField jField = new JField(access, name, desc, signature, value);
        List<IFieldConfigurator> fieldConfigurators = new ArrayList<IFieldConfigurator>();
        for (IClassConfigurator classConfigurator : getCommonConfigurators()) {
            fieldConfigurators.add(classConfigurator.createFieldConfigurator(jField));
        }
        return new ScanFieldVisitor(fieldConfigurators);
    }


    /**
     * call configurationComplete on each configurator.
     */
    @Override
    public void visitEnd() {
        for (IClassConfigurator classConfigurator : getCommonConfigurators()) {
            classConfigurator.configurationComplete();
        }
    }
}
